function outcome = camb_mech_points(varargin)

% This function computes outcomes under the Cambridge Mechanism. 
% It works in three different ways:
% 1) If passed 7 arguments, it computes an allocation and return market clearing cuttoffs for each program
% 2) If passed 4 arguments, it computes the probability of assigment for each Rank order list
% 3) If passed 5 arguments, it computes the probability of justified envy.
bd = @(x) max(min(x,1),0);

switch nargin
    case 7 % Get pivotal cutoff

        [sch_ranks, paidlunch, sch_seats, pl_seats, fl_seats, priorities, lotteries] ...
                        = deal(varargin{:});
        % Get assignments
        [assignments,~] = camb_mech_assignments(sch_ranks, paidlunch, sch_seats, pl_seats, fl_seats, priorities, lotteries);
        NPROG =numel(sch_seats);
        NROWS =size(sch_ranks,1);
        
        % Report specific priority for each participant
        sch_ranks(isnan(sch_ranks) | sch_ranks ==0 )=NPROG+1;
        tot_prior= cambridge_RSP(sch_ranks,priorities,lotteries);
        
        % Outcome: pivotal cutoff - find the last student accepted in each program
        outcome = zeros(2*NPROG,1);
        for S = 1:NPROG
            excess_cap_sch = sch_seats(S) - sum(assignments==S);
            excess_cap_pl = pl_seats(S) - sum(assignments==S & paidlunch);
            excess_cap_fl = fl_seats(S) - sum(assignments==S & ~paidlunch);
            plp = max((excess_cap_sch==0)*min(tot_prior(assignments==S,S)),...
                            (excess_cap_pl==0)*min(tot_prior(assignments==S & paidlunch,S)));
            
            flp = max((excess_cap_sch==0)*min(tot_prior(assignments==S,S)),...
                            (excess_cap_fl==0)*min(tot_prior(assignments==S & ~paidlunch,S)));
                            
            if numel(plp)>0, outcome(S) = plp; else, outcome(S) = 40 * (pl_seats(S)==0 | sch_seats(S)==0); end
            if numel(flp)>0, outcome(S+NPROG) = flp; else, outcome(S+NPROG) = 40 * (fl_seats(S)==0  | sch_seats(S)==0); end
        end
    
    case  4 % Get probability of assignment for each possible choice
        [sch_ranks, paidlunch, priorities, cutoffs] ...
                        = deal(varargin{:});
        NPROG =numel(priorities);
        NROWS =size(sch_ranks,1);
        sch_ranks(isnan(sch_ranks) | sch_ranks ==0 )=NPROG+1;
        tot_prior= cambridge_RSP(sch_ranks,repmat(priorities,NROWS,1),1);
        
        % Select relevant cutoffs
        
        cutoffs = cutoffs(:,NPROG*(1-paidlunch)+(1:NPROG));
        
        % Outcome: Average probability of assignment
        outcome = zeros(NROWS,NPROG);
        accum = zeros(size(cutoffs,1),1);
        for R = 1:NROWS
            accum = accum*0;
            for k = 1:3
                sch  = sch_ranks(R,k);
                if sch > NPROG,
                    prb = 1 - accum;
                else
                    prb = bd(bd(tot_prior(R,sch) - cutoffs(:,sch)) - accum);
                end
                outcome(R,sch) = mean(prb);  % Mean over cutoff realizations
                accum = accum + prb;
            end
        end
    
    case 5 % Get justified envy
        [sch_ranks, paidlunch, priorities, cutoffs,v] ...
                        = deal(varargin{:});
        NPROG =numel(priorities);
        NROWS =size(sch_ranks,1);
        sch_ranks(isnan(sch_ranks) | sch_ranks ==0 )=NPROG+1;
        tot_prior= cambridge_RSP(sch_ranks,repmat(priorities,NROWS,1),1);
        % Select relevant cutoffs

        cutoffs = cutoffs(:,NPROG*(1-paidlunch)+(1:NPROG));
        
        % Outcome: instances of justified envy
        outcome=zeros(NROWS,1);
        % Potential Justified Envy
        bare_cutoffs = mod(floor(cutoffs),10); % Clean up lottery draws and rank priority
        pot_je = bsxfun(@gt,priorities,bare_cutoffs); % Somebody with lower priority got assigned.
        % What is the probability that the student gets assigned to a less preferred school?
        % Justified Envy
        accum = zeros(size(cutoffs,1),1);
        for R = 1:NROWS
            accum = accum*0; accum_je = 0;
            v_je = max(bsxfun(@times,v(R,:),pot_je),[],2); % Maximum utility in a school with potential je
            for k = 1:3
                sch  = sch_ranks(R,k);  % School in ROL R postion k
                if sch>NPROG, continue; end
                % This agent has justified envy if it is assigned to this school and
                % somebody with lower priority got assigned to a more preferred school.
                je =(v(R,sch)<v_je); 
                prb = bd(bd(tot_prior(R,sch) - cutoffs(:,sch)) - accum);
                accum_je = accum_je + mean(prb.*je);
                accum = accum + prb;
            end
            outcome(R) = accum_je + mean((1-accum).*(0<v_je)); % Chance of remaining unassigned.
        end  
end


function tot_prior= cambridge_RSP(sch_ranks,priorities,lotteries)
    NROWS = size(sch_ranks,1);
    NPROG = size(priorities,2);
    % Compute total priorities
    % priorities: 0 no priority, 1 proximity priority, 2 sibling priority and 3 proximity+sibling priority 
    tot_prior= bsxfun(@plus,priorities,lotteries);
    tot_prior= [tot_prior zeros(size(tot_prior,1),1)];
    tot_prior= tot_prior + sparse(1:NROWS,sch_ranks(:,1),30,NROWS,NPROG+1) + ...
                           sparse(1:NROWS,sch_ranks(:,2),20,NROWS,NPROG+1) + ...
                           sparse(1:NROWS,sch_ranks(:,3),10,NROWS,NPROG+1) - 10;
    tot_prior= max(tot_prior(:,1:NPROG),0);                   
    % For example: 22.22 means that the cutoff occurs at a student that ranked j 1st,
    % had sibling priority and lottery tie-breaker 0.22.
    % 40 means that the school was not available (see code below).
    % 0 means that noone was rejected. 
    % 10.1 means that the cutoff occurs at a student that ranked j 2nd, no priority and
    % tie-break 0.1;
    % 0.43 means that the cutoff occurs at a student that ranked j 3rd, no priority and
    % tie-break 0.43.