
% Runs the DA counterfactuals

% Clear all except do_draws and do_specs
clearvars -except do_draws do_specs do_overwrite tasks;

% Basic Manual settings
do_value_sim     = 1000;  % draws from the estimated distribution of valuations
do_cutoffs_draws = 1000;  % draws from the cutoff distribution
if ~exist('do_draws','var'), do_draws = 0:1000; end % default if run_draws doesn't exist; data and bootstrap samples
if ~exist('do_specs','var'), 
    do_specs = {'truthful','current_year','last_year','current_year_av','current_year_wl','mixture_naive','mixture_soph'}; 
end 
% Computational options
delete(gcp('nocreate'));
parpool('local',16);

% Optional settings to save time with recomputations;
if ~exist('do_overwrite','var'), do_overwrite = true; end % default: true

% Fixed Settings 

addpath ../estimation;
addpath ../est_lotteries;
addpath ../boston_code;

% Iterate over specs and draws
for do_draw_num = do_draws
for do_spec = do_specs, 
    clearvars -except do_* tasks
    spec = do_spec{1};
    fprintf('Starting: Draw %d specification: %s\n',do_draw_num,spec)
    draw_tic = tic;
    
    % Set up options for wrap_DA_cutoffs
    if do_draw_num > 0
        save_disk_space = true;
        str_draw_num = num2str(do_draw_num);
        str_folder = 'bootstrap/';
        cutoffs_file = 'cutoffs_ts_bootstrap'; draw_field = 'draw';
    else
        save_disk_space = false;
        str_draw_num = '';
        str_folder = '';
        cutoffs_file = 'cutoffs_ts_bootstrap'; draw_field = 'estm';
    end
    
    % Spec variables
    truthful_behavior       = false; 
    bootstrap_for_inference = true;
    switch spec
        case 'truthful',        est_type = 'truthful1';          bootstrap_for_inference = false;   truthful_behavior = true;
        case 'current_year',    est_type = 'currentyear1_ts'; 
        case 'current_year_av', est_type = 'currentyearav1_ts';  method = 'current_year_av_ts';
        case 'current_year_wl', est_type = 'currentyear1_ts_wl'; method='current_year_ts_wl';
        case 'last_year',       est_type = 'lastyear1_ts';       method='last_year_ts';
        case 'mixture_naive',   est_type = 'mixture1_ts';        truthful_behavior = true;
        case 'mixture_soph',    est_type = 'mixture1_ts';
    end
    
    % Decide whether to run this draw/spec.
    if truthful_behavior, tr = '_truthful_'; else tr='_'; end
    savefname=['mat/' est_type tr num2str(do_draw_num)];
    if ~do_overwrite && (exist([savefname '.mat'],'file')==2),
        load(savefname);
        if save_disk_space && exist('pref_da_camb','var') && exist('frac_je','var'), continue; end
        if ~save_disk_space && exist('V_da','var') && exist('V_camb','var') && exist('JE_camb','var'), continue; end
    end
    
    % Load parameters
    folder = ['../estimation/' est_type];
    if do_draw_num > 0  
        if bootstrap_for_inference  % Get parameters from bootstrap sample do_draw_num
            fname = [folder '/param_A_' num2str(do_draw_num) '.mat'];
            if exist(fname, 'file')==2
                load(fname,'mean_B_vec','mean_Sigma'); B=mean_B_vec; S=mean_Sigma;
            else
                continue            
            end
        else                        % Get parameters from a posterior draw
            load([folder '/chain_random.mat'],'B','S'); B=B(:,do_draw_num); S=squeeze(S(:,:,do_draw_num));
        end
    else                            % Get point estimates
        load([folder '/chain_summ_X.mat'],'mean_B','mean_S'); B=mean_B; S=mean_S;
    end
    
    
    % Load objective assignment probabilities
    load(['../est_lotteries/' str_folder 'current_year_ts' str_draw_num],'lott','student_type','smpl');    
    as.lott  = lott; as.student_type = student_type; as.smpl = smpl;
    
    % Load behavioral or subjective assignement probabilities
    bh = as;   % default
    if do_draw_num > 0
        switch spec
        case 'current_year_av', 
            load('data','year'); current_year_av_ts; 
            bh.lott=lott; bh.student_type=student_type;
        case 'current_year_wl',
            load('data','asgn','rank','year','regs','paidlunch'); current_year_ts_wl; 
            bh.lott=lott; bh.student_type=student_type; as=bh;
        case 'last_year',
            [bh.lott,smpl,bh.student_type]=gen_lott_file(['../est_lotteries/' cutoffs_file],do_draw_num,1);
        end
    else
        if exist('method','var'), bh=load(['../est_lotteries/' method]); end    
    end
        
    % Load simulated cutoffs
    cutoffs_camb =load(['../est_lotteries/' cutoffs_file],draw_field); cutoffs_camb = cutoffs_camb.(draw_field);
    if numel(size(cutoffs_camb))==5, cutoffs_camb = cutoffs_camb(:,do_draw_num,:,:); end
    cutoffs_camb=squeeze(cutoffs_camb); % cutoffs_camb is now an array of market clearing cutoffs (num_years,#cutoff draws,programs)
    if size(cutoffs_camb,2) > do_cutoffs_draws; 
        rng(1); cutoffs_camb =  cutoffs_camb(:,randsample(size(cutoffs_camb,2),do_cutoffs_draws),:);  % get a random set of cutoffs draws;
    end
    
    % Run the wrapper
    wrap_DA_cutoffs;
    
    fprintf('Finished Draw %d of spec %s in %d seconds\n',do_draw_num,spec,toc(draw_tic));    
    
end
end

delete(gcp('nocreate'));