function [ final_vals, priors, run_next_list] = gen_chain_mcmc_mex(folder,priors,init_vals,chain,run_this_list,NITER_by_file,stem,thin_each)
%GEN_CHAIN_MCMC Generates a MCMC chain using the gibbs sampler.
%   It loads the master.mat file in 'folder', and the 'priors' and
%   'init_vals' to run an MCMC using gibbs_sampler_test_sur_v. It splits
%   the chain in files [stem chain j.mat] for j in run_this_list.
%   The main output is the stored file, but it also returns the variables
%   that are useful to continue the chain.
GPU_if_Available =false;

runGPU = false;
if GPU_if_Available
    try 
        D = gpuDevice;
        runGPU = true; 
    end
end

if nargin<8, thin_each=1; end % Do not thin file if thin_each is not included.
% Load master file and transform Xcell in X if needed.
X=[];
load([folder '/master.mat']);

% Construct X, if necessary.
if isempty(X), X = blkdiag(Xcell{:}); end
n_coeff = size(X, 2);

% Drop students that are not included in the sample
constraint_matrix=constraint_matrix(esample);
distances = distances(esample,:);
V=V(esample,:);
X=X(repmat(esample(:),n_progs,1),:);

% Diagnose Errors
assert(sum(sum(isnan(X))) + sum(sum(isinf(X))) == 0); %infs or nans in X
assert(sum(sum(isnan(distances))) + sum(sum(isinf(distances))) == 0); %infs or nans in distances
assert(sprank(X)==min(size(X))); % Full rank design matrix;


% Set deault values if not provided.
if nargin<2 ||isempty(priors),
    priors.b0 = zeros(n_coeff, 1);
    priors.var0 = 100 * eye(n_coeff);
    priors.nu = 100;
    priors.V_wishart = eye(n_progs);
end
if nargin<3 || isempty(init_vals), init_vals.V=V; end
if nargin<4 || isempty(chain) , chain=0; end
if nargin<5 || isempty(run_this_list), run_this_list=[]; end
if nargin<6 || isempty(NITER_by_file), NITER_by_file=1000; end
if nargin<7 || isempty(stem), stem='chain_'; end

% Resume option: if init_vals is an integer, then load 
% [folder '/' stem num2str(chain) '_run_' num2str(init_vals)] 
if isreal(init_vals) && numel(init_vals)==1 && mod(init_vals,1)==0 % If we are resuming.
    resumeAt= init_vals; run_this_new_list = run_this_list(run_this_list>resumeAt);
    load([folder '/' stem num2str(chain) '_run_' num2str(resumeAt)], 'final_vals'); init_vals=final_vals;
    [final_vals, priors] = gen_chain_mcmc_mex(folder,priors,init_vals,chain,run_this_new_list,NITER_by_file,stem,thin_each);
    
else % If we are not resuming
    % Run the MCMC using the Gibbs sampler.
    final_vals = init_vals;

    if numel(run_this_list)>0
        fprintf('Ready to run chain %d!\n',chain);
    else
        fprintf('Returning initial values and  default priors\n');
    end    

    for savenum = run_this_list;
        if runGPU
		   %save('AC_noadd','X', 'constraint_matrix', 'priors', 'distances', 'NITER_by_file', 'final_vals')
           tic; [all_B_vec, all_Sigma, all_V, final_vals,init_vals] = gibbs_sampler_gpu(X, constraint_matrix, priors, [], distances, NITER_by_file, final_vals); toc;
        else 
           tic; [all_B_vec, all_Sigma, all_V, final_vals,init_vals] = gibbs_sampler_mex(X, constraint_matrix, priors, [], distances, NITER_by_file, final_vals); toc;
        end
		
        filenameSave = [folder '/' stem num2str(chain) '_run_' num2str(savenum) '.mat'];
        
        
		if thin_each>1 % Thin files
			thin = 1:thin_each:1000;
			all_B_vec=all_B_vec(:,thin);
			all_Sigma=all_Sigma(:,:,thin);
			all_V=all_V(:,:,thin);
			fprintf('%s\n',filenameSave);
		end
			
		save(filenameSave, 'all*','*_vals');
		fprintf('%s Saved\n',filenameSave);
		
        
    
    end
end
run_next_list = run_this_list + numel(run_this_list);

end

