function [all_B_vec, all_Sigma, all_V, all_N, all_Ng, final_vals, init_vals] = gibbsSamplerMixMex(X, constraint_matrix, priors, V, distances, NITER, init_vals,groups)
% X: start with a cell array (Xcell) of X matrices for each equation
% 	 * Xcell{1} is a matrix of covariates for each student, corresponding 
%	   to program 1.  Each row is a student, and each column is a covariate
%    Then, X = blkdiag(Xcell{:});  i.e., it is the stacked matrix of Xs
%
% constraint_matrix: cell array where c...{s} is the matrix A for student s
%                    so that Av > 0
% priors: structure with the following fields --
%         * b0: n_coeff x 1 
%  	  * var0: n_coeff x n_coeff 
%	  * nu: number of degrees of freedom of the inverse Wishart
%	  * V_param: other parameter of the inverse Wishart
% V: n_students x n_progs matrix of utilities in the cone given by A -- to initialize
% distances: n_students x n_progs matrix of distances from student to program
% NITER: the number of iterations to run
% 
% init_vals: optional structure for initial values of B and Sigma and the state of the random generator.
% 	    * If this is not provided, the initial values of B and Sigma will
%	      be drawn from the prior distribution (and not recorded)
%	    * This is used if you want to split up the run into smaller runs of
%	      1000 iterations each, say, so that gigantic matrices need not be
%	      stored in memory.
%	    * Optional Fields: Sigma (initial value of Sigma); B_vec (row vector for initial value of B_vec); and rng_state (state of rng) V (initial V's, if not provided as a fourth argument);
%
% Requires: kronmult.m and phi.m (mex file)

% Specify whether you want to use kronmult for the matrix multiplication or not
% If this is a 0 (i.e., you want to use parfor), then make sure you uncomment
% all the lines for parallelization
USE_KRONMULT = 0;

vec_op = @(ZZ)(ZZ(:));

n_students = size(constraint_matrix,1);
n_coeff = size(X, 2); 
n_progs = size(X, 1) / n_students;

% Create some matrices for later
mat1 = zeros(n_students, n_progs, n_coeff);
mat2 = zeros(n_students, n_progs);
student_location = repmat((1:n_students)', n_progs, 1);

student_data = cell(n_students, 1);
for s = 1:n_students
    student_data{s} = sparse(X(student_location == s, :));
end

% Priors
b0_vec = priors.b0; %  mean of prior on beta
var0 = priors.var0; % 
inv_var = inv(var0);

nu = priors.nu; % number of degrees of freedom of the inverse Wishart
V_param = priors.V_wishart; % other parameter of inverse Wishart

alpha = priors.alpha;
beta = priors.beta;
% Loading initial values, if supplied.
if nargin <= 6 || ~isfield(init_vals,'rng_state')
   rng(0);
   init_vals.rng_state=rng;
   fprintf('Resetting the random seed... The initial state will be saved in init_vals.rng_state\n')
else
   rng(init_vals.rng_state);
end

if nargin <= 6 || ~isfield(init_vals,'Sigma')
   Sigma = iwishrnd(V_param, nu);
   init_vals.Sigma=Sigma;
else
   Sigma=init_vals.Sigma;
end

if nargin <= 6 || ~isfield(init_vals,'B_vec')
   B_vec = mvnrnd(b0_vec, var0); % VECTOR = vec(beta matrix)
   init_vals.B_vec=B_vec;
else
   B_vec=init_vals.B_vec;
end

if ~(nargin <= 6) &&  isfield(init_vals,'V')
    if numel(V)>0,
		fprintf('I am IGNORING the V supplied in the init_vals structure.\n This field is used only if V is set to [].\n');
		init_vals.V=V;
	else
		V = init_vals.V;
    end
end
N = init_vals.N;

% Define output variables;
all_Sigma = zeros(n_progs, n_progs, NITER);
all_B_vec = zeros(n_coeff, NITER);
all_V = zeros(n_students, n_progs, NITER);
all_N = zeros(n_students, NITER);
all_Ng = zeros(max(groups),NITER);
% Q helps with a Kronecker product down the line
Q = cell(2, 1);
Q{2} = eye(n_students);


%index_vec = (0:(n_progs-1))' * n_students; % This is the vector to read off indices for mu from all_mu
%VOLD = V;
for nval = 1:NITER
    if mod(nval, 100) == 0
       fprintf('\n Iteration %d:', nval);
    end

    % First update the distributions for beta and Sigma
    U = chol(Sigma);
    Uinv = inv(U);
    Q{1} = Uinv';

    % Now left multiply V = X*beta + eps by (U^-1)' to normalized error term's variance
    V_vec = vec_op(V + distances); % V is students x programs; recall that coeff on distances is normalized to -1
    if USE_KRONMULT == 1
       Xtilde = kronmult(Q, X);
       V_vec_tilde = kronmult(Q, V_vec);
    else
       vr = reshape(V_vec, n_students, n_progs);
	   %save_rng=rng;
       for s = 1:n_students
       	  a = Uinv' * student_data{s};
		  b = Uinv' * vr(s, :)';
	      mat1(s, :, :) = a(:, :);
	      mat2(s, :) = b(:);
       end
       Xtilde = reshape(mat1, n_students*n_progs, n_coeff);
       V_vec_tilde = reshape(mat2, n_students * n_progs, 1);
    end
	%rng(save_rng);
    % Formulas for Bayesian OLS
    inv_posterior_var = (Xtilde' * Xtilde + inv_var);
    beta_tilde = inv_posterior_var \ (Xtilde' * V_vec_tilde + inv_var * b0_vec);
    B_vec = mvnrnd(beta_tilde, inv(inv_posterior_var)); % New draw of beta

    % Update the distribution of the variance matrix
    error_vec = V_vec - X*B_vec';
    error_mat = reshape(error_vec, numel(error_vec) / n_progs, n_progs);
    Smat = error_mat' * error_mat;
	Sigma = iwishrnd(Smat + V_param, nu + n_students); % New draw of the variance matrix
    
    % In each iteration, we first loop through the students and draw new
    % values of v for them
    
    % Values that are constant across students
    mat_cell = cell(n_progs,1);
    cond_sd_vec = zeros(n_progs, 1);
    
    for p = 1:n_progs
        S12 = Sigma(p, :);
        S12(p) = [];
        
        S22 = Sigma;
        S22(p, :) = [];
        S22(:, p) = [];
        
        mat_cell{p} = S12 / S22; % This is S12 * inv(S22); Used in conditional mean
        cond_sd_vec(p) = sqrt(Sigma(p, p) - S12 * inv(S22) * S12'); % This is the conditional variance
    end
    all_mu = X * B_vec';
    all_mu = reshape(all_mu, n_students, n_progs);
    % The draws of student's valuations are a transformation of the following uniform draws;
	
	probNaive = zeros(max(groups),1);
    for g=1:max(groups)
        alphapos = alpha + sum(N(groups==g)==2); betapos = beta +  sum(N(groups==g)==1);
        probNaive(g) = betarnd(alphapos,betapos);
    end
    pN = probNaive(groups);
	draws_mat = rand(n_students,n_progs);
    

    %[V1,N1]=sampleUtilsOriginalMix(V,constraint_matrix,all_mu,distances,draws_mat,mat_cell,cond_sd_vec,pN(:));
	
    [V, N]=sampleUtilsMixMexOpt(V, constraint_matrix, all_mu, distances, draws_mat, mat_cell, cond_sd_vec, pN(:));
	
    % Save the draws
    all_Sigma(:, :, nval) = Sigma(:, :);
    all_B_vec(:, nval) = B_vec(:);
    all_V(:, :, nval) = V(:, :);
	all_N(:,nval) = N;
    all_Ng(:,nval) = probNaive(:);
    
    if nval<6
       fprintf('(%0.3f,%0.3f) ', pN(1),pN(2));
    end

end


final_vals.rng_state = rng;
final_vals.Sigma = Sigma;
final_vals.B_vec = B_vec;
final_vals.V = V;
final_vals.N = N;

%a
