function [savedSumFiles,savedThinFiles] = read_mcmc(folder,chains,diagnosis,thinV,thinAll)
% READ_CHAIN_MCMC reads MCMC chains and stores a summary file and (if requested) a thinned version of the chain. 
% Syntax savedFiles = read_chain_mcmc(folder,chains,diagnosis,thin)
% folder: is the folder where the chains are.
% chains: points at the chain to be read. It can be a structure or a cell of structures. 
%       The structures should have the following fields: stem, chain, list. 
%		We'll load [folder '/' stem num2str(chain) '_' num2str(num)], where num is an
%		element of list.
% diagnosis: 1 to perform diagnosis.
% thin: take 1 each 'thin' draws.
% savedSumFiles is a cell with the set of summary files saved.
% savedThinFiles is a cell with the set of thin files saved.



%% Definining the parameters
%folder= 'results3'
def.stem = 'chain_';
def.chain = '0';
def.list = 25:50;

if ischar(chains), def.stem=chains; end
if ismatrix(chains),def.list = chains; end
if isstruct(chains), def=chains; end
if ~iscell(chains), chains={def}; end

if (~isstruct(diagnosis)) && diagnosis>0,
    diagnosis = struct();
    diagnosis.on=1;    
    diagnosis.Coda_justScreen = 1;
    diagnosis.Geweke_justScreen = 1;
else
    if isstruct(diagnosis), diagnosis.on=1;
    else diagnosis = struct('on',false);
    end
end

%Thinning
if nargin>=4 && ~isempty(thinV)
	if isstruct(thinV), thinV=thinV.each; end
else
	thinV=0;
end
if nargin<5, thinAll=1; end	
savedSumFiles={};	
savedThinFiles={};	
allThins={};
allChains={};
%% Loading files;
B_all = [];
for chn = 1:numel(chains)
	ch = chains{chn};
	% Thinning
	if thinV>0
		[B,S,V]=readchain([folder '/' ch.stem],ch.chain,ch.list,thinV,thinAll);
		th = ch;
        th.B = B;
		th.S = S;
		th.V = V;
		th.thin = thinV;
		fileSave=[ch.stem 'thin' num2str(thinV) '_' num2str(ch.chain)];
		save([folder '/' fileSave], '-struct', 'th')
		savedThinFiles = cat(1,savedThinFiles,{fileSave});
	else
		[B,S]=readchain([folder '/' ch.stem],ch.chain,ch.list,thinV,thinAll);
	end
    B_all{chn}=B;
    S_all{chn}=S;
    
	[n_parm_b,n_draws] = size(B);
	ch.numsim = size(B,2);
	ch.mean_B = mean(B,2);
	ch.mean_S = mean(S,3);
	error2_B=(B - repmat(ch.mean_B,1,ch.numsim)).^2; 
	error2_S=(S - repmat(ch.mean_S,1,1,ch.numsim)).^2;
	ch.mean_error2_B=mean(error2_B,2);
	ch.mean_error2_S=mean(error2_S,3);
    
    
    sqchl = floor(sqrt(n_draws));  use=1+n_draws-sqchl^2:n_draws;
     % These are the last sqchl^2 draws.
	[~,num_sd2_B] = btch_sig(B(:,use),sqchl,2);
    ch.ind_draws_B= var(B(:,use),[],2)'./num_sd2_B * (n_draws/sqchl^2); % ind_draws = RNE * sqchl^2. (Notice that var(B) is not var(B)/n).
    [~,num_sd2_error2_B] = btch_sig(error2_B(:,use),sqchl,2);
    ch.ind_draws_error2_B= var(error2_B(:,use),[],2)'./num_sd2_error2_B * (n_draws/sqchl^2);
    % Sigma
    [~,num_sd2_S] = btch_sig(S(:,:,use),sqchl,3);
    ch.ind_draws_S= var(S(:,:,use),[],3)./reshape(num_sd2_S,size(S,1),size(S,2));
    [~,num_sd2_error2_S] = btch_sig(error2_S(:,:,use),sqchl,3);
    ch.ind_draws_error2_S= var(error2_S(:,:,use),[],3)./reshape(num_sd2_error2_S,size(S,1),size(S,2));   
	fileSave=[ch.stem 'summ_' num2str(ch.chain)];
	if diagnosis.on
        fprintf(''); % Insert breakpoint here to plot stuff interactively.
        if isfield(diagnosis,'Taceplots') && diagnosis.Traceplots
            %Traceplots
            coeffs = 1:3; plot(B(coeffs,:)');
            coeffs = 1:3; coeffs2 = 1; plot(squeeze(S(coeffs,coeffs2,:))');
        end
         if isfield(diagnosis,'RunninMeans') && diagnosis.RunningMeans
            %RunningMeans
            rmB=cumsum(B,2)./repmat([1:ch.numsim],n_parm_b,1);
            coeffs = 1:3; plot(rmB(coeffs,:)');
            rmS=cumsum(S,3)./permute(repmat([1:numsim],size(S,1),1,size(S,2)),[1,3,2]);
            coeffs = 1:3; coeffs2 = 1; plot(squeeze(rmS(coeffs,coeffs2,:))');
            diagnosis.RunninMeans.rmS=rmS;
            diagnosis.RunninMeans.rmB=rmB;
         end
         if isfield(diagnosis,'YuMykland') && diagnosis.YuMykland
             cd('mcmcdiag');
             C =cusum(B'); 
             coeffs = 1:3; plot(C(:,coeffs));
             cd('..')
             diagnosis.RunninMeans=C;
         end
         if isfield(diagnosis,'RafteryLewis') && diagnosis.RafteryLewis
             cd('gibbsdiag');
             rl = raftery(B',0.025,0.005,0.95);
             rqn = zeros(n_parm_b,1); for j=1:n_parm_b, rqn(j)=rl(j).n; end; srgn=sort(rqn);
             fprintf('[Min Median Mean Max] Draws required: %d,%d,%d,%d. Draws computed: %d \n',...
                 min(srgn),srgn(round(n_parm_b/2)),round(mean(srgn)),max(srgn),size(B,2));
             cd('..')
             diagnosis.RafteryLevis = rl;
         end
         if isfield(diagnosis,'Coda') && diagnosis.Coda
             cd('gibbsdiag');
             info.r= 0.025; info.q=0.1; info.s=0.95; info.p1=0.2; info.p2=0.5; 
             diagnosis.coda=coda(B',[],info);
             cd('..')
              
         end
        if isfield(diagnosis,'Coda_justScreen') && diagnosis.Coda_justScreen
             cd('gibbsdiag');
             info.r= 0.025; info.q=0.1; info.s=0.95; info.p1=0.2; info.p2=0.5; 
             coda(B',[],info);
             cd('..')
        end
    ch.diagnosis=diagnosis;
    end
    save([folder '/' fileSave], '-struct','ch')
	savedSumFiles = cat(1,savedSumFiles,{fileSave});
	ch.B= B; ch.S = S;
	
    %allThins=cat(1,allThins,th);
	allChains=cat(1,allChains,ch);


end

save([folder '/all_coefs.mat'],'S_all','B_all')
fprintf('Saved all_coefs\n');

% Multiple chains;
numChains = numel(allChains);
Jch.numsim = 0;
Jch.mean_B = 0;
Jch.mean_S = 0;
Jch.mean_error2_B=0;
Jch.mean_error2_S=0;X={};
if numChains>1
	for chn = 1 : numChains;
		ch=allChains{chn};
		new_numsim= Jch.numsim +ch.numsim;
		Jch.mean_B = (Jch.mean_B*Jch.numsim + ch.mean_B*ch.numsim)/new_numsim;
		Jch.mean_S = (Jch.mean_S*Jch.numsim + ch.mean_S*ch.numsim)/new_numsim;
		Jch.mean_error2_B=(Jch.mean_error2_B*Jch.numsim + ch.mean_error2_B*ch.numsim)/new_numsim;
		Jch.mean_error2_S=(Jch.mean_error2_S*Jch.numsim + ch.mean_error2_S*ch.numsim)/new_numsim;
		Jch.numsim = new_numsim;
		
		if diagnosis.on
			 % X is a NxDxM matrix which contains M MCMC  simulations of length N, each with dimension D.
			 X{chn} = [ch.B'] % reshape(chain.S,size(chain.S,1)*size(chain.S,2),size(chain.S,3))'];
		end
	end
	
	if diagnosis.on
        cd('mcmcdiag');
		% Gelman-Rubin Potential Scale Reduction Factor;
		if isfield(diagnosis,'GelmanRubin') && diagnosis.GelmanRubin
        	[R,neff,Vh,W,B,tau,thinX] = psrf(X{:});
            diagnosis.GelmanRubin.R = R;
            diagnosis.GelmanRubin.neff = neff;
            diagnosis.GelmanRubin.Vh = Vh;
            diagnosis.GelmanRubin.W = W;
            diagnosis.GelmanRubin.B = B;
            diagnosis.GelmanRubin.tau = tau;
            diagnosis.GelmanRubin.thinX =thinX;
        end
		if isfield(diagnosis,'GelmanRubinM') && diagnosis.GelmanRubinM
        % Multivariate Potential Scale Reduction Factor;
		[R,neff,V,W,B] = mpsrf(X{:});
            diagnosis.GelmanRubinM.R = R;
            diagnosis.GelmanRubinM.neff = neff;
            diagnosis.GelmanRubinM.V = V;
            diagnosis.GelmanRubinM.W = W;
            diagnosis.GelmanRubinM.B = B;
        end
        cd('..');
        if isfield(diagnosis,'Geweke') && diagnosis.Geweke
		% Geweke;
        cd('gibbsdiag')
		info.p1=allChains{1}.numsim/(allChains{1}.numsim+allChains{end}.numsim); 
		info.p2=allChains{end}.numsim/(allChains{1}.numsim+allChains{end}.numsim);
        diagnosis.Geweke = coda([X{1};X{end}],[],info);
        cd('..');
        end
        if isfield(diagnosis,'Geweke_justScreen') && diagnosis.Geweke_justScreen
		% Geweke;
        cd('gibbsdiag')
		info.p1=allChains{1}.numsim/(allChains{1}.numsim+allChains{end}.numsim); 
		info.p2=allChains{end}.numsim/(allChains{1}.numsim+allChains{end}.numsim);
        coda([X{1};X{end}],[],info);
        cd('..');
        end
        Jch.diagnosis = diagnosis;
    end
    fileSave=[ch.stem 'summ_X'];
	save([folder '/' fileSave], '-struct','Jch')
	savedSumFiles = cat(1,savedSumFiles,{fileSave});
	
             
		
end
	



